"======================================================================
|
|   Smalltalk GUI base class for widget wrappers with publish/subscribe
|
|
 ======================================================================"

"======================================================================
|
| Copyright 2004 Mike Anderson
| Written by Mike Anderson
|
| This file is part of Mumble, a development environment for GNU Smalltalk.
|
| Mumble is free software; you can redistribute it and/or modify it
| under the terms of the GNU General Public License as published by the Free
| Software Foundation; either version 2, or (at your option) any later version.
| 
| Mumble is distributed in the hope that it will be useful, but WITHOUT
| ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
| FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
| details.
| 
| You should have received a copy of the GNU General Public License along with
| Mumble; see the file COPYING.  If not, write to the Free Software
| Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
|
 ======================================================================
"

Namespace current: GTK.GUI!

GTK.GUI.WindowedView subclass: #WindowView
	instanceVariableNames: 'wnd ebox title visible geom popupMenu'
	classVariableNames: 'windows'
	poolDictionaries: ''
	category: 'GUI-Framework'!

!GTK.GUI.WindowView class methodsFor: 'quitting'!

closeAll
    windows copy do: [ :each | each close: self ].
! 
!

!GTK.GUI.WindowView class methodsFor: 'instance creation'!

new
	| r |
	r := super new.
    windows isNil ifTrue: [ windows := Set new ].
    windows add: r.
	(windows size = 1) ifTrue: [ Gtk newDispatcher ].
	^r
!	

!GTK.GUI.WindowView methodsFor: 'initialization'!

buildWidgets    
    super buildWidgets.
    wnd := GtkWindow new: Gtk gtkWindowToplevel.
	
	"So that we can record our screen position, to restore it
	 on image restart."
    wnd on: 'configure-event' send: #configureEvent:event: to: self.
	wnd on: 'destroy' send: #destroyEvent to: self.  
    popupMenu := nil.
!

attachViews
	self child ifNotNil: [ :c | wnd add: c external ].
!

!GTK.GUI.WindowView methodsFor: 'accessing'!

external
    ^wnd
!

close
    self close: self
!

close: aSender
    wnd destroy.
!

closeAll
    self class closeAll.
!

defaultHeight: aHeight
    wnd propertiesAt: 'default-height' put: aHeight
!

defaultWidth: aWidth
    wnd propertiesAt: 'default-width' put: aWidth
!

show
    wnd show.
	visible := true.
!

title: aTitle
    wnd setTitle: aTitle.
	title := aTitle.
!

!GTK.GUI.WindowView methodsFor: 'events'!

destroyEvent
    windows remove: self ifAbsent: 
		[ ('Internal error: Window ', wnd getTitle printString, 
			' was not in the window list.') printNl ].
    windows isEmpty ifTrue: 
		[ Gtk stopDispatcher.
		"Browser stopDead." ].
!

configureEvent: aSender event: aEvent
    | ev |
    "Previously I used #castTo: here, but it now seems to be ignored, and 
	    #changeClassTo: seems to do the trick."
    ev := aEvent.
    ev := (aEvent isKindOf: GdkEventConfigure) 
		ifFalse: [ ev changeClassTo: GdkEventConfigure ].
    (ev isKindOf: GdkEventConfigure) 
		ifFalse: [ 'cast failed??' printNl ].
    geom := Rectangle 
		left: ev x value 
		right: ev x value + ev width value
		top: ev y value
		bottom: ev y value + ev height value.
!

applicationName
	^'GUI'
!

snapshotCallback
	ObjectMemory snapshot: 
		(self applicationName, '.im').
!
!

!GTK.GUI.WindowView methodsFor: 'saving and loading'!

restoreNonStateProperties
	super restoreNonStateProperties.
	self restoreGeometry.
	title ifNotNil: [ :t | wnd setTitle: t ].
	visible ifNotNil: [ :v | v ifTrue: [ wnd show ] ].
!

restoreGeometry
    geom isNil ifTrue: [ ^self ].
    wnd move: geom left y: geom top.
    wnd resize: geom width height: geom height.
!
!

!GTK.GUI.WindowView methodsFor: 'testing'!

isWindow
    ^true
!
!

!WindowView methodsFor: 'childViews and parentViews'!

windowView
	^self.
!

child
    (childViews notNil and: [ childViews notEmpty ])
		ifTrue: [ ^childViews anyOne ].
    ^nil.
!

add: aChildView
	super add: aChildView.
	self attachViews.
!
!

!WindowView methodsFor: 'popup menus'!

popupMenu: aItemsBlock widget: aWidget button: aButton time: aActivateTime
	| prev |
	"aButton is expected to be 3, but we have to pass it to Gtk, and it is
	 laborious to create it manually, so we may as well pass it around."

	"We hang onto the popup menu so that it doesn't get destroyed at
	 an inopportune time, but we can't reuse it because we might have 
	 moved some of the items to another popup menu when another view 
	 in the same window activated a popup menu. We don't actually 
	 #add: it to ourself, though, because it is ephemeral."

	prev := popupMenu.
	"If you embed an EventBox within an EventBox (for example), you will get
	 multiple button callbacks, so avoid rebuilding the menu unnecessarily."
	prev notNil ifTrue: 
		[ prev lastPopupTime = aActivateTime ifTrue: [ ^self ] ].
	
	popupMenu := (PopupMenuView new)
		triggerWidget: aWidget;
		buildMenuFrom: aItemsBlock value;
		yourself.
	prev notNil ifTrue: [ prev destroy. ].
	popupMenu
		popupButton: aButton
		activateTime: aActivateTime.
!
!

!WindowView class methodsFor: 'convenience'!

safeTitleString: aString
	| s r |
	aString ifNil: [ ^'' ].
	s := aString contractTo: 75.
	s := s reject: [ :ch | ch asciiValue < 32 ].
	^s
!
!
